precision highp float;

varying vec2 uTexCoord;

uniform sampler2D prevTexture;
uniform sampler2D nextTexture;
uniform sampler2D mapTexture;

uniform float uAmount;
uniform float uSensitivity;
uniform float uFrameScale;

uniform float u0;
uniform float u1;
uniform float u2;
uniform float u3;
uniform float u4;
uniform float u5;

uniform vec2 uResolution;

#define PI 3.1415926535898
#define SQRT_2_PI 2.5066282746310

#define MEAN 0.0
#define STD_DEVIATION 1.0

float gaussian_distribute(float mean, float std_dev, float x) {
    float result = 1.0 / (SQRT_2_PI * std_dev) * exp(-pow(x - mean, 2.0) * 0.5 / pow(std_dev, 2.0));
    return result;
}

float gaussian_distribute_2d(float mean, float std_dev, vec2 v) {
    float x = v.x;
    float y = v.y;
    float std_dev_2 = pow(std_dev, 2.0);
    float result = 0.5 / (PI * std_dev_2)
                    * exp(-(pow(x - mean, 2.0) - 2.0 * (x - mean) * (y - mean) + pow(y - mean, 2.0)) * 0.5 / std_dev_2);
    return result;
}

bool inside(vec2 pos, vec4 region) {
    return step(region.x, pos.x) * step(pos.x, region.z) * step(region.y, pos.y) * step(pos.y, region.w) == 1.0;
}

vec2 re_map(vec2 m) {
    return mix(smoothstep(vec2(0.0), vec2(0.1), m) * 0.1,
                smoothstep(vec2(0.1), vec2(1.0), m) * 0.9 + 0.1,
                step(vec2(0.1), m));
}

void main() {
    vec2 map = texture2D(mapTexture, uTexCoord).rg / uFrameScale;
    // 需要引入delta_time（表示两帧时间间隔程度，间隔越久，该值越大，从外面传入），
    // 以降低相邻帧不同时间差对运动模糊的影响，否则高帧率视频模糊效果会很弱。
    vec2 off_delta = normalize(map) * 2.0 / uResolution;
    vec4 prev_color = texture2D(prevTexture, uTexCoord/* - map*/);
    float max_edge = max(uResolution.x, uResolution.y);
    float sensitivity = uSensitivity * 0.001;
    map = abs(map) / max_edge;

    float invert_sensitivity = 0.1 - sensitivity;
    map = clamp(vec2(invert_sensitivity), vec2(u1), map);
    map = (map - invert_sensitivity) / (u1 - invert_sensitivity);
    map = smoothstep(vec2(0.0), vec2(0.5), map);
    map = pow(map, vec2(u2)) * u2;
    map = map * u0;
    float l_map = length(map);

    vec4 color = vec4(0.0);
    vec2 offset = vec2(0.0);
    float weight = 0.0;
    float weight_f = 0.0;
    float x = 0.0;
    float count = 0.0;
    float std_dev = u3 * l_map + 0.001;

    while (count < uAmount) {
        weight_f = gaussian_distribute(MEAN, std_dev, x);
        color += texture2D(nextTexture, uTexCoord + offset) * weight_f;
        offset += off_delta;
        weight += weight_f;
        x += u4;
        count += 1.0;
    }

    weight_f = gaussian_distribute(MEAN, std_dev, x);
    color += texture2D(nextTexture, uTexCoord + offset) * weight_f;
    weight += weight_f;

    float blend = u5 / max(1.0, l_map);
    gl_FragColor = mix(color / weight, prev_color, blend);
}


