precision highp float;

varying highp vec2 uTexCoord;

uniform int uInterpolate;
uniform highp float uRatio;

uniform sampler2D prevYTexture;
uniform sampler2D prevUVTexture;
uniform sampler2D nextYTexture;
uniform sampler2D nextUVTexture;
uniform sampler2D mapTexture;

uniform highp vec2 uResolution;

#define PI 3.1415926535898

vec4 yuv2rgb(sampler2D luminance, sampler2D chrominance, vec2 uv) {
    vec3 yuv;
    yuv.x = texture2D(luminance, uv).r - 1.0/16.0;
    yuv.yz = texture2D(chrominance, uv).ra - vec2(0.5, 0.5);
    return vec4(
        yuv.x * 1.164 + yuv.y * 0.0 + yuv.z * 1.793,
        yuv.x * 1.164 + yuv.y * -0.213 + yuv.z * -0.533,
        yuv.x * 1.164 + yuv.y * 2.112 + yuv.z * 0.0,
        1.0
    );
}

void main() {
    if (uInterpolate == 1) {
        vec2 map = texture2D(mapTexture, uTexCoord).rg / uResolution;
        vec4 prev_color = yuv2rgb(prevYTexture, prevUVTexture, uTexCoord + map * uRatio);
        vec4 next_color = yuv2rgb(nextYTexture, nextUVTexture, uTexCoord - map * (1.0 - uRatio));

        float blend = -(cos(PI * uRatio) - 1.0) / 2.0;
        gl_FragColor = prev_color * (1.0 - blend) + next_color * blend;
    } else {
        gl_FragColor = yuv2rgb(prevYTexture, prevUVTexture, uTexCoord);
    }
}


